#
# Tables structure:
#
# We did not use the MySQL AUTO_INCREMENT feature
# since it's not portable.
#

# Global Parameters:
DROP TABLE IF EXISTS t_ulk_parms;
CREATE TABLE t_ulk_parms (
	par_name	CHAR(32) NOT NULL PRIMARY KEY,
	par_value	VARCHAR(64) NOT NULL DEFAULT ""
);

# Users are identifier by an internal ID and and a logical name
# which is usually their login name.
# Event for huge companies, which can have multiple Windows Domains
# we assume that user login names are unique
#
DROP TABLE IF EXISTS t_ulk_user;
CREATE TABLE t_ulk_user (
	usr_id		INT NOT NULL PRIMARY KEY,
	usr_name	VARCHAR(32) NOT NULL DEFAULT "",
	usr_fullname	VARCHAR(128) NOT NULL DEFAULT "",
	usr_email	VARCHAR(128) NOT NULL DEFAULT "",
	usr_lang	CHAR(2) NOT NULL DEFAULT "fr",
	usr_accinfo	VARCHAR(250) NOT NULL DEFAULT "",
	usr_init	TINYINT(1) NOT NULL DEFAULT 0
);

# Users can have multiple account:
DROP TABLE IF EXISTS t_ulk_usraccount;
CREATE TABLE t_ulk_usraccount (
	usr_id		INT NOT NULL,
	act_id		INT NOT NULL,
	PRIMARY KEY(usr_id,act_id)
);

# This table contains the script to run
# when a password must be changed:
# it also describes the different category
DROP TABLE IF EXISTS t_ulk_genaccount;
CREATE TABLE t_ulk_genaccount (
	act_id		INT NOT NULL PRIMARY KEY,
	act_name	CHAR(32) NOT NULL DEFAULT "",
	act_desc	VARCHAR(250) NOT NULL DEFAULT "",
	act_script	VARCHAR(250) NOT NULL DEFAULT ""
);

# The script must be located under the ./scripts directory
# It is called with the following parameters:
# ../scripts/${act_script} usr_name new_passwd

# This table memorizes the mandatory questions:
DROP TABLE IF EXISTS t_ulk_admquery;
CREATE TABLE t_ulk_admquery (
	adq_id		INT NOT NULL,
	adq_lang	CHAR(2) NOT NULL DEFAULT "fr",
	adq_text	VARCHAR(254) NOT NULL DEFAULT "",
	PRIMARY KEY(adq_id,adq_lang)
);

# This table memorizes the user answers:
DROP TABLE IF EXISTS t_ulk_admanswer;
CREATE TABLE t_ulk_admanswer (
	usr_id		INT NOT NULL,
	adq_id		INT NOT NULL,
	adq_answer	VARCHAR(250) NOT NULL DEFAULT "",
	PRIMARY KEY(usr_id,adq_id)
);

# This table memorizes user questions:
DROP TABLE IF EXISTS t_ulk_usrquery;
CREATE TABLE t_ulk_usrquery (
	usr_id		INT NOT NULL,
	usq_id		INT NOT NULL,
	usq_query	VARCHAR(250) NOT NULL DEFAULT "",
	usq_answer	VARCHAR(250) NOT NULL DEFAULT "",
	PRIMARY KEY(usr_id,usq_id)
);

# This table memorizes some statistics:
DROP TABLE IF EXISTS t_ulk_stat;
CREATE TABLE t_ulk_stat (
	usr_id		INT NOT NULL,
	sta_client	INT NOT NULL,
	sta_date	TIMESTAMP,
	sta_action	INT NOT NULL,
	sta_info	VARCHAR(250) NOT NULL DEFAULT ""
);

# sta_action can be:
#	1 = PASSWORD CHANGED
#	2 = LOGIN
#	3 = TICKET GENERATION
#	4 = WRONG AUTH when ask for password change

# log table (handles error and trace):
DROP TABLE IF EXISTS t_ulk_log;
CREATE TABLE t_ulk_log (
	log_date	TIMESTAMP,
	log_client	INT NOT NULL,
	log_cat 	INT NOT NULL,
	log_nu		INT NOT NULL,
	log_info	VARCHAR(250) NOT NULL DEFAULT ""
);

# log_cat can be:
#	1 = ERROR
#	2 = WARNING
#	3 = INFO

# This table is used when user's are asked to initialize
# their account: they receive a message with an opaque value
# that allow them to be identified without giving their
# ID & password
DROP TABLE IF EXISTS t_ulk_init;
CREATE TABLE t_ulk_init (
	usr_id		INT NOT NULL,
	ini_session	VARCHAR(250) NOT NULL,
	ini_date	TIMESTAMP,
	PRIMARY KEY(ini_session)
);

# EOF
