<?php
	/**
	 * Extends text-only email class to allow multipart html & plain-text email
	 *
	*/
	function quoted_printable_encode_character($matches) {
		$character = end($matches);
		return sprintf('=%02x',ord($character));
	}

	class MailManPlus extends MailMan {
		var $htmlMessage;
		var $_boundary = "MultiPart-Boundary";
			

		function quoted_printable_encode($string) {
			$string = preg_replace_callback(
					'/[^\x21-\x3C\x3E-\x7E\x09\x20]/',
					'quoted_printable_encode_character',
					$string);

			$newline = "=\n";

			preg_match_all('/.{73}([^=]{0,3})/',$string,$match);
			
			# add the last line block that may be smaller than 73 chars:
			$match[0][] = substr($string,strlen(implode("",$match[0])));
			
			return implode($newline,$match[0]);
		}

		/**
		 * Sets the HTML code part of the email.
		 * You can use a template if you want just like you can
		 * for <code>setTextMessage</code>
		 *
		 * @param str htmlMsg HTML source code
		 * @see setTextMessage
		 */
		function setHTMLMessage($htmlMsg){
			$this->htmlMessage = $this->quoted_printable_encode($htmlMsg);
		}
		
		/**
		 * Envelopes the multipart contents of the email. Syntax here is crucial, including line breaks.
		 * 
		 * @access private
		 * @return str Combined multiparts of the message body
		 */
		function _getCombinedParts(){
			$br = $this->br;
			$message = "--".$this->_boundary.$br;
			$message .= "Content-Type: text/plain;charset=\"iso-8859-1\"$br";
			#$message .= "Content-Transfer-Encoding: quoted-printable$br$br";
			$message .= $this->textMessage;
			$message .= "$br$br";
			$message .= "--".$this->_boundary.$br;
			if (!empty($this->htmlMessage)) {
				$message .= "Content-Type: text/html;charset=\"iso-8859-1\"$br";
				$message .= "Content-Transfer-Encoding: quoted-printable$br$br";
				$message .= $this->htmlMessage;
				$message .= "$br$br";
				$message .= "--".$this->_boundary."--".$br.$br;
			}
			return $message;
		}
		
		/**
		 * Adds multipart message header, defines multipart boundary. This method overrides the parent class'.
		 *
		 * @access private
		 */
		function _makeContentHeaders(){
			$this->_headers[] = "Content-Type: Multipart/Alternative;boundary=\"$this->_boundary\"";
		}
		
		/**
		 * Sends the message.
		 *
		 * @see sendMessage API for MailMan
		 */
		function sendMessage(){
			//Safeguards to make sure everything needed to send is present
			if (count($this->from) < 1 || count($this->to) < 1 || !isset($this->subject)){
				die("You must provide at least valid sender,receipient and subject");
			}
			if (!isset($this->textMessage) && !isset($this->htmlMessage)){
				die("You must set both TEXT and HTML message parts to use MailManPlus");
			}
			
			$hdr = $this->_makeHeaders();
			
			//combine multiparts
			$message = $this->_getCombinedParts();
			
			//Creates To recipients string. CC and BCC recipient strings are handled as additional headers			
			$recipients = $this->_createRecipientString($this->to);
			
			//send message
			mail($recipients,$this->subject,$message,$hdr);

			//send kickback message if any
			if (isset($this->kickback)){
				echo $this->kickback;
			}
		}
		
		/**
		 * @see sendMessage API for MailMan
		 */
		function showEmail(){
			$email = "Subject: " . $this->subject . $this->br;
			$email .= "To: " . $this->_createRecipientString($this->to) . $this->br;
			$email .= "From: " . $this->_createRecipientString($this->from) . $this->br;
			$email .= $this->_makeHeaders();
			$email .= $this->_getCombinedParts();
			return $email;
		}
		//==========================================
	}
?>
