<?php
#
# This script is called by the Web Interface (populate2.php) to
# fill the database from an uploaded file.
# But it can be called manually with a CSV file as a parameter.
#
# When called, the following variables must be set:
#	$userfile = name of file to parse
#	$test = 1 for testing only
#
# Output variables are:
#	$gMSG = last error
#	$errors = array of error strings
#	$ok = 1 if no error occurs
#

include_once "locale.inc.php";
include_once "backend.inc.php";
include_once "utils.inc.php";

$gMSG = "";
$errors = array();
$newusers = array ();
$ok = 1;

if (($fh = @fopen($userfile,"r"))) {
	if (!($accounts = BackendGetAllGenAccount())) {
		$gMSG = _("Erreur: aucun type de compte n'est d&eacute;fini");
		$ok = 0;
	}
	else {
		$accountlist = array();
		for ( $idx = 0 ; $idx < count($accounts) ; $idx++ ) {
			$accountlist[$accounts[$idx]['act_name']] = $accounts[$idx]['act_id'];
		}

		$linenu = 1;
		while ($line = fgets($fh,2048)) {
			CheckLine($linenu++,trim($line),$errors);
		}
		@fclose($fh);
	}
}
else {
	$gMSG = _("Erreur &agrave; l'ouverture du fichier");
	$ok = 0;
}

# Check if errors occurs and what to do then:
if (count($errors)) {
	$gMSG = _("Corrigez les erreurs et recommencez !");
	$ok = 0;
}
elseif (!$test) {
	if (($fh = @fopen($userfile,"r"))) {
		$linenu = 1;
		while ($line = fgets($fh,2048)) {
			AddLine($linenu++,trim($line),$errors);
		}
		@fclose($fh);
	}
	else {
		$gMSG = _("Erreur &agrave; l'ouverture du fichier");
		$ok = 0;
	}

	if ($ok) {
		$linenu--;
		$gMSG = _('$linenu nouveaux utilisateurs ajout&eacute;s &agrave; la base !');
		$gMSG = preg_replace('/\$linenu/',$linenu,$gMSG);
	}
}

# End of process

function CheckLine($linenu,$line,&$errors)
{
	global $accountlist;
	global $newusers;
	global $gUsrLangSupported;
	$ml = _('Ligne $linenu: ');
	$ml = preg_replace('/\$linenu/',$linenu,$ml);

	# Split the line in parts:
	# name,email,account type,additional account type....
	$parts = explode(";",$line);

	$user = array_shift($parts);
	$fullname = array_shift($parts);
	$email = array_shift($parts);
	$lang = array_shift($parts);

	if (!isset($user) || empty($user)) {
		$mg = _('l\'utilisateur \'$user\' est vide');
		$mg = preg_replace('/\$user/',$user,$mg);
		array_push($errors,$ml.$mg);
		return;
	}

	if (!isset($email) || empty($email)) {
		$mg = _('l\'adresse mail de \'$user\' est vide');
		$mg = preg_replace('/\$user/',$user,$mg);
		array_push($errors,$ml.$mg);
		return;
	}

	if ($row = BackendGetUserByName($user)) {
		$mg = _('l\'utilisateur \'$user\' existe d&eacute;j&agrave;');
		$mg = preg_replace('/\$user/',$user,$mg);
		array_push($errors,$ml.$mg);
	}
	else {
		# Check if the user is not already specified in the file:
		if (in_array($user,$newusers)) {
			$mg = _('l\'utilisateur \'$user\' existe plusieurs fois dans le fichier');
			$mg = preg_replace('/\$user/',$user,$mg);
			array_push($errors,$ml.$mg);
		}
		else {
			array_push($newusers,$user);
		}
	}

	# Check the specified language:
	if (!in_array($lang,$gUsrLangSupported)) {
		$mg = _('la langue \'$lang\' n\'est pas support&eacute;e pour l\'utilisateur \'$user\'');
		$mg = preg_replace('/\$lang/',$lang,$mg);
		$mg = preg_replace('/\$user/',$user,$mg);
		array_push($errors,$ml.$mg);
	}

	for ( $idx = 0 ; $idx < count($parts) ; $idx++ ) {
		$account = $parts[$idx];
		if (empty($account)) {
			if ($idx == 0) {
				$mg = _('aucun type de compte n\'est d&eacute;fini pour l\'utilisateur \'$user\'');
				$mg = preg_replace('/\$user/',$user,$mg);
				array_push($errors,$ml.$mg);
			}
			continue;
		}

		if (!isset($accountlist[$account])) {
			$mg = _('le type de compte \'$account\' pour l\'utilisateur \'$user\' n\'existe pas');
			$mg = preg_replace('/\$account/',$account,$mg);
			$mg = preg_replace('/\$user/',$user,$mg);
			array_push($errors,$ml.$mg);
		}
	}
}

function AddLine($linenu,$line,&$errors)
{
	global $accountlist;
	global $accounts;

	# Split the line in parts:
	# name,email,account type,additional account type....
	$parts = explode(";",$line);

	$user = array_shift($parts);
	$fullname = array_shift($parts);
	$email = array_shift($parts);
	$lang = array_shift($parts);
	$info = "";

	if (!($usr_id = BackendNewUser($user,$fullname,$lang,$info,$email))) {
		$mg = _('l\'utilisateur \'$user\' existe d&eacute;j&agrave;');
		$mg = preg_replace('/\$user/',$user,$mg);
		array_push($errors,$ml.$mg);
	}

	for ( $idx = 0 ; $idx < count($parts) ; $idx++ ) {
		$account = $parts[$idx];
		if (empty($account))
			continue;

		if (!isset($accountlist[$account])) {
			$mg = _('le type de compte \'$account\' pour l\'utilisateur \'$user\' n\'existe pas');
			$mg = preg_replace('/\$user/',$user,$mg);
			$mg = preg_replace('/\$account/',$account,$mg);
			array_push($errors,$ml.$mg);
		}
		if (BackendAddUserAccount($usr_id,$accountlist[$account]) != 1) {
			$mg = _('erreur lors du rattachement du compte \'$account\' &agrave, l\'utilisateur \'$user\'');
			$mg = preg_replace('/\$user/',$user,$mg);
			$mg = preg_replace('/\$account/',$account,$mg);
			array_push($errors,$ml.$mg);
		}
	}
}
?>
