<?php
#
# This script is called by the Web Interface (ticketinit2.php) to
# create the initial tickets.
#
# When called, the following variables must be set:
#	$test = 1 for testing only
#	$nomail = 1 to avoid email sending
#	$alluser = 1 to send a ticket to all users who need one
#	$userlist = list of users who need a ticket
#
# Output variables are:
#	$errors = array of error strings
#	$report = array of info about generated tickets
#	$ok = 1 if no error occurs
#
include_once "locale.inc.php";
include_once "backend.inc.php";
include_once "utils.inc.php";
include_once "mailman.class.php";
include_once "mailmanplus.class.php";

$gMSG = "";
$errors = array();
$report = array();
$msg    = array();
$ok = 1;

# For log generation:
if (!isset($_SERVER['REMOTE_ADDR']))
	$cliaddr = ip2long("127.0.0.1");
else
	$cliaddr = $_SERVER['REMOTE_ADDR'];

# Get all the user who have not fill their interface.
# For each of them, we generate a ticket.
# The ticket is mailed to the user so they can fill their interface.

$nu = 0;
$users = array ();

# Build the list of target users:
if ($alluser) {
	$users = BackendGetAllUserID();
}
else {
	$ul = explode("\n",$userlist);
	foreach ($ul as $u) {
		$u = rtrim($u);
		if (empty($u)) continue;
		$usr = BackendGetUserByName($u);
		if (!$usr) {
			$mg = _('Le nom \'$u\' n\'est pas un nom d\'utilisateur valide');
			$mg = preg_replace('/\$u/',$u,$mg);
			array_push($report,$mg);
		}
		else {
			$users[] = $usr;
		}
	}
}

for ( $idx = 0 ; $idx < count($users) ; $idx++ ) {
	$usr_id = $users[$idx]['usr_id'];

	# If users are given by the list: always generate a ticket:
	if ($alluser && BackendIsUserAccountInit($usr_id)) {
		continue;
	}

	# Generate a ticket if necessary:
	if (1 || !BackendGetTicket($usr_id)) {
		$user = BackendGetUser($usr_id);
		$name = $user['usr_name'];

		if (!$test) {
			$sid = BackendAddTicket($usr_id);
			$mg = _('Ticket $sid cr&eacute;&eacute; pour l\'utilisateur \'$name\'');
			$mg = preg_replace('/\$sid/',$sid,$mg);
			$mg = preg_replace('/\$name/',$name,$mg);
			array_push($report,$mg);
		}
		else {
			$mg = _('Le compte de l\'utilisateur \'$name\' n\'est pas initialis&eacute;');
			$mg = preg_replace('/\$name/',$name,$mg);
			array_push($report,$mg);
		}

		if (!$test && !$nomail) {
			$mail = new MailManPlus();
			$email = $user['usr_email'];
			$mail->setFrom($GLOBALS['gAdmEmail'],$GLOBALS['gAdmEmail']);
			$mail->addRecipient($name,$email);
			$lang = $user['usr_lang'];
			$subject = $GLOBALS['gMessageSubject'][$lang];
			$mail->setSubject($subject);

			if (!isset($msg["text_$lang"])) {
				$filename = $GLOBALS['gMessageTEXTFile'][$lang];
				$msg["text_$lang"] = @file_get_contents($GLOBALS['gMessageDir']."/$lang/$filename");
				if (!$msg["text_$lang"]) {
					$msg["text_$lang"] = "";
					$mg = _('Impossible d\'ouvrir le fichier \'$filename\'');
					$mg = preg_replace('/\$filename/',$filename,$mg);
					array_push($errors,$mg);
					# exit the loop gracefully:
					$idx = count($users) + 1;
				}
			}

			# Perform substitutions:
			$text_body = preg_replace('/%L%/',$GLOBALS['gSiteName']."/".$GLOBALS['gAppName']."/user/userprefs.php?sessionid=$sid",$msg["text_$lang"]);
			$text_body = preg_replace('/%U%/',$user['usr_fullname'],$text_body);
			$text_body = preg_replace('/%u%/',$name,$text_body);
			$text_body = preg_replace('/%E%/',$email,$text_body);
			$text_body = preg_replace('/%A%/',$GLOBALS['gAdmEmail'],$text_body);

			if (!isset($msg["html_$lang"])) {
				$filename = $GLOBALS['gMessageHTMLFile'][$lang];
				$msg["html_$lang"] = @file_get_contents($GLOBALS['gMessageDir']."/$lang/$filename");
				if (!$msg["html_$lang"]) {
					$msg["html_$lang"] = "";
					$mg = _('Impossible d\'ouvrir le fichier \'$filename\'');
					$mg = preg_replace('/\$filename/',$filename,$mg);
					array_push($errors,$mg);
					# exit the loop gracefully:
					$idx = count($users) + 1;
				}
			}

			# Perform substitutions:
			$html_body = preg_replace('/%L%/',$GLOBALS['gSiteName']."/".$GLOBALS['gAppName']."/user/userprefs.php?sessionid=$sid",$msg["html_$lang"]);
			$html_body = preg_replace('/%U%/',$user['usr_fullname'],$html_body);
			$html_body = preg_replace('/%u%/',$name,$html_body);
			$html_body = preg_replace('/%E%/',$email,$html_body);
			$html_body = preg_replace('/%A%/',$GLOBALS['gAdmEmail'],$html_body);

			if ($GLOBALS['gMessageTEXTEncoding']) 
				$mail->setTextMessage($text_body);
			if ($GLOBALS['gMessageHTMLEncoding']) 
				$mail->setHTMLMessage($html_body);

			#echo $mail->showEmail();
			$mail->sendMessage();
			$mg = _('Ticket g&eacute;n&eacute;r&eacute; et mail envoy&eacute; pour l\'utilisateur \'$name\'');
			$mg = preg_replace('/\$name/',$name,$mg);
			array_push($report,$mg);

			# Add a stat:
			$info = _('G&eacute;n&eacute;ration du ticket $sid');
			$info = preg_replace('/\$sid/',$sid,$info);
			BackendAddStat($usr_id,$cliaddr,ULK_ACTION_GENTICKET,$info);
	
			$nu++;
		}
	}
}

# Generate a log:
$info = _('G&eacute;n&eacute;ration de $nu tickets');
$info = preg_replace('/\$nu/',$nu,$info);

BackendLog($cliaddr,ULK_TYPE_INFO,ULK_INF_GENTICKET,$info)
?>
