/*
 * ===========================================================================
 * Fichier: log.c
 * ---------------------------------------------------------------------------
 * Date:	04/02/06
 * Auteur :	J.Delamarche 
 * Version:	1.0
 * ---------------------------------------------------------------------------
 * Gestion des logs.
 * ===========================================================================
 */
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <tchar.h>
#include "log.h"
#include "syncunlock.h"

static SYNCUNLOCK_LOGCBACK	g_logcback = NULL;
static HANDLE	g_eh = NULL;	/* handle pour le ReportEvent */
static _TCHAR	*g_szAppName = _T("syncunlock");

/*
 * ---------------------------------------------------------------------------
 * Mise en place d'une fonction callback pour les messages d'erreur.
 * En fonctionnement normal, il n'y a pas de callback: tous les messages
 * vont dans le log systme. Pour tester avec testlotus1, on redirige ainsi
 * les messages directement  l'cran.
 * ---------------------------------------------------------------------------
 */
void WINAPI LogSetCallback(SYNCUNLOCK_LOGCBACK cback)
{
	g_logcback = cback;
}

/*
 * ---------------------------------------------------------------------------
 * Initialisation gnrale des logs:
 * ---------------------------------------------------------------------------
 */
DWORD LogInit(HANDLE hModule)
{
	HKEY	hkey;
	DWORD	status = NOERROR;
	DWORD	dwTypes;
	DWORD	dwCatNu;
	_TCHAR	szPath[MAX_PATH];

	/* Vrifie que l'application s'est bien enregistre comme source: */
	_stprintf(szPath,_T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\%s"),g_szAppName);

	status = RegCreateKey(HKEY_LOCAL_MACHINE,szPath,&hkey);
	GetModuleFileName(hModule,szPath,sizeof(szPath));

	/* Enregistre le EventMessageFile: */
	status = RegSetValueEx(hkey,_T("EventMessageFile"),0,REG_EXPAND_SZ,
					(PBYTE)szPath,(_tcslen(szPath)+1) * sizeof(TCHAR));

	/* Enregistre le nombre de catgories supportes: */
	dwCatNu = 1;
	status = RegSetValueEx(hkey,_T("CategoryCount"),0,REG_DWORD,
					(LPBYTE)&dwCatNu,sizeof(dwCatNu));

	/* Enregistre le CategoryMessageFile: */
	status = RegSetValueEx(hkey,_T("CategoryMessageFile"),0,REG_EXPAND_SZ,
					(PBYTE)szPath,(_tcslen(szPath)+1) * sizeof(TCHAR));

	/* Enregistre les types d'vnements: */
	dwTypes = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
	status = RegSetValueEx(hkey,_T("TypesSupported"),0,REG_DWORD,
					(LPBYTE)&dwTypes,sizeof(dwTypes));

	/* Pour simplifier, nous n'enregistrons pas de catgorie.... */
	RegCloseKey(hkey);

	return 0;
}

/*
 * ---------------------------------------------------------------------------
 * Fonction d'initialisation des logs par thread:
 * ---------------------------------------------------------------------------
 */
DWORD LogThreadInit(HANDLE hModule)
{
	/* Se dfinit comme une source d'vnements: */
	g_eh = RegisterEventSource(NULL,g_szAppName);

	return 0;
}

/*
 * ---------------------------------------------------------------------------
 * Fonction de fermeture gnrale des logs:
 * ---------------------------------------------------------------------------
 */
DWORD LogEnd(HANDLE hModule)
{
	return 0;
}

/*
 * ---------------------------------------------------------------------------
 * Fonction de fermeture des logs par thread:
 * ---------------------------------------------------------------------------
 */
DWORD LogThreadEnd(HANDLE hModule)
{
	return DeregisterEventSource(g_eh);
}

/*
 * ---------------------------------------------------------------------------
 * Fonction de chargement d'un message  partir du fichier de ressources:
 * ---------------------------------------------------------------------------
 */
UINT MyLoadMessage(DWORD dwMsgId, PTSTR pszBuffer, UINT cchBuffer, ...)
{	
	va_list	args;
	va_start(args,cchBuffer);

	return FormatMessage(
			FORMAT_MESSAGE_FROM_HMODULE,
			NULL,
			dwMsgId,
			LANG_NEUTRAL,
			pszBuffer,
			cchBuffer,
			&args
		);
}

 /*
 * ---------------------------------------------------------------------------
 * Fonction d'extraction d'un message systme associ  la dernire erreur
 * remonte par l'OS (celle retourne par GetLastError()).
 *
 * Attention: le pointeur retourn doit tre libr par LocalFree() !
 * ---------------------------------------------------------------------------
 */
LPCTSTR GetSystemMessage()
{	
	LPCTSTR lpMsgBuf;
	
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);

	/* Ne pas oublier de librer ce buffer ! */
	return lpMsgBuf;
}

/*
 * ---------------------------------------------------------------------------
 * Fonction de rapport d'erreur dans le Event Log:
 * ---------------------------------------------------------------------------
 */
void MyReportEvent(
				   WORD	wType,			/* message type: ERROR ou INFO */
				   const _TCHAR *msg		/* message  envoyer dans le log */
				  )
{
	LPCTSTR lpStrings[1] = { msg };
	WORD	wCatID;
	DWORD	dwEventID;

	if (wType == EVENTLOG_INFORMATION_TYPE) {
		wCatID = SYNCUNLOCK_CATEG1;
		dwEventID = GENERIC_INFO1;
	}
	else {
		wCatID = SYNCUNLOCK_CATEG1;
		dwEventID = GENERIC_ERROR1;
	}

	if (g_logcback) {
		(*g_logcback)(msg);
	}
	else {
		ReportEvent(
				g_eh,
				wType,
				wCatID,			/* category */
				dwEventID,		/* event ID */
				NULL,			/* User's SID */
				1,
				0,
				lpStrings,
				NULL);
	}
}

/*
 * ---------------------------------------------------------------------------
 * Fonction de gnration des logs (attention: il faut les mettre dans un 
 * fichier, pas  l'cran !)
 * ---------------------------------------------------------------------------
 */
void Log(
		 WORD wType,				/* type d'erreur: INFO ou ERROR (voir doc ReportEvent) */
		 const TCHAR *szFormat,		/* format de l'erreur */
		 ...)						/* paramtres additionnels */
{
	_TCHAR buffer[1024];
	FILE *out;

	va_list	args;
	va_start(args,szFormat);

	_vsntprintf(buffer,sizeof(buffer),szFormat,args);
	buffer[sizeof(buffer)/sizeof(TCHAR)-1] = _T('\0');

#if _DEBUG
	//printf("%s\n",buffer);
#endif

	if ((out = fopen(_T("c:\\tmp\\output.log"),"a"))) {
		fprintf(out,_T("%s\n"),buffer);
		fclose(out);
	}

	MyReportEvent(wType,buffer);

	va_end(args);
}

/* EOF */