/*
 * ===========================================================================
 * Fichier: parm.c
 * ---------------------------------------------------------------------------
 * Date:	04/02/06
 * Auteur :	J.Delamarche
 * Version:	1.0
 * ---------------------------------------------------------------------------
 * Les fonctions de ce module permettent de grer l'interface avec la base
 * de registre qui stocke les paramtres externes de l'application.
 * ===========================================================================
 */
#include <malloc.h>
#include <string.h>
#include <tchar.h>
#include "log.h"
#include "parm.h"

/*
 * ---------------------------------------------------------------------------
 * Retourne une structure contenant tous les paramtres externes 
 * de l'application. 
 *
 * Cette fonction est super-moche.... on aurait pu mieux faire...
 * ---------------------------------------------------------------------------
 */
DWORD GetParms(
			   lsParm *parm,			/* structure qui va recevoir les paramtres */
			   void (*logFunc)(WORD wType, const TCHAR *szFormat, ...)	/* gestionnaire des erreurs */
			  )
{
	DWORD	status;
	LPCTSTR	lpMsg;
	HKEY	hkey; 
	DWORD	dwType;
	DWORD	dwSize;
	TCHAR	*szMyKey = _T("SOFTWARE\\Trickytools\\syncunlock");
	TCHAR	*szVal;

	/* valeur par dfaut: */
	memset(parm,0,sizeof(lsParm));
	parm->m_log   = logFunc;

	/* Lit les valeurs des paramtres dans la base de registres: */
	if ((status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,szMyKey,
		(DWORD)NULL,KEY_ALL_ACCESS,&hkey)) != ERROR_SUCCESS) {
		SetLastError(status);
		(*logFunc)(EVENTLOG_ERROR_TYPE,_T("RegOpenKeyEx(%s) a chou: %s"),
							szMyKey,lpMsg = GetSystemMessage());
		LocalFree((HLOCAL)lpMsg);
		return status;
	}

	/* Paramtre "mode debug": */
	szVal = _T("DebugMode");
	dwSize = sizeof(parm->m_debug);
	if ((status = RegQueryValueEx(hkey,szVal,
						NULL,&dwType,(LPBYTE)&parm->m_debug,&dwSize)) != ERROR_SUCCESS) {
		SetLastError(status);
		(*logFunc)(EVENTLOG_ERROR_TYPE,_T("RegQueryValueEx(%s) a chou: %s"),
							szVal,lpMsg = GetSystemMessage());
		LocalFree((HLOCAL)lpMsg);
		return status;
	}
	if (parm->m_debug)
		(*logFunc)(EVENTLOG_INFORMATION_TYPE,_T(">>>> Debug Mode activ"));

	/* Paramtre "mode active": */
	szVal = _T("Active");
	dwSize = sizeof(parm->m_active);
	if ((status = RegQueryValueEx(hkey,szVal,
						NULL,&dwType,(LPBYTE)&parm->m_active,&dwSize)) != ERROR_SUCCESS) {
		SetLastError(status);
		(*logFunc)(EVENTLOG_ERROR_TYPE,_T("RegQueryValueEx(%s) a chou: %s"),
							szVal,lpMsg = GetSystemMessage());
		LocalFree((HLOCAL)lpMsg);
		return status;
	}
	if (parm->m_debug)
		(*logFunc)(EVENTLOG_INFORMATION_TYPE,_T("Active Mode=%d"),parm->m_active);

	/* Path de la commande externe: */
	szVal = _T("ExternalCommand");
	dwSize = sizeof(parm->m_szExternalCommand);
	if ((status = RegQueryValueEx(hkey,szVal,
						NULL,&dwType,parm->m_szExternalCommand,&dwSize)) != ERROR_SUCCESS) {
		SetLastError(status);
		(*logFunc)(EVENTLOG_ERROR_TYPE,_T("RegQueryValueEx(%s) a chou: %s"),
							szVal,lpMsg = GetSystemMessage());
		LocalFree((HLOCAL)lpMsg);
		return status;
	}
	if (parm->m_debug)
		(*logFunc)(EVENTLOG_INFORMATION_TYPE,_T("External Command=%s"),parm->m_szExternalCommand);

	/* Secret partag pour le chiffrement MD5: */
	szVal = _T("SharedSecret");
	dwSize = sizeof(parm->m_szSharedSecret);
	if ((status = RegQueryValueEx(hkey,szVal,
						NULL,&dwType,parm->m_szSharedSecret,&dwSize)) != ERROR_SUCCESS) {
		SetLastError(status);
		(*logFunc)(EVENTLOG_ERROR_TYPE,_T("RegQueryValueEx(%s) a chou: %s"),
							szVal,lpMsg = GetSystemMessage());
		LocalFree((HLOCAL)lpMsg);
		return status;
	}
	if (parm->m_debug)
		(*logFunc)(EVENTLOG_INFORMATION_TYPE,_T("Shared Secret=%s"),parm->m_szSharedSecret);

	RegCloseKey(hkey);

	return 0;
}

/* EOF */