/*
 * ===========================================================================
 * Fichier: syncunlock.c
 * ---------------------------------------------------------------------------
 * Date:	04/02/06
 * Auteur :	J.Delamarche
 * Version:	1.0
 *
 * ---------------------------------------------------------------------------
 * Point d'entre de la DLL qui synchronise les mots de passe Windows
 * avec Unlockator
 *
 * La description des fonctions ci-dessous est impose par Microsoft car elles
 * doivent suivre les spcifications de PASSFILT.DLL pour tre appeles par
 * le PDC.
 *
 * NOTE: le nom des DLL de filtrage ne doit pas dpasser 8 caractres !!!!
 *	     (sinon elles ne seront pas charges au dmarrage - W2003)
 *
 * ===========================================================================
 */
#include <windows.h>

#include <ntsecapi.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <tchar.h>
#include <lmaccess.h>
#include <lmapibuf.h>
#include "parm.h"
#include "process.h"
#include "log.h"

#ifndef STATUS_SUCCESS
#define STATUS_SUCCESS ((NTSTATUS)0x0L)
#endif

/*
 * --------------------------------------------------------------------------
 * Point d'entre (voir doc Microsoft)
 * --------------------------------------------------------------------------
 */
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//Log(EVENTLOG_INFORMATION_TYPE,_T("syncunlock.dll called !"));

	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			LogInit(hModule);
			LogThreadInit(hModule);
			Log(EVENTLOG_INFORMATION_TYPE,_T("syncunlock.dll charge !"));
		case DLL_THREAD_ATTACH:
			break;
		case DLL_PROCESS_DETACH:
			LogThreadEnd(hModule);
			LogEnd(hModule);
		case DLL_THREAD_DETACH:
			break;
    }
    return TRUE;
}

/*
 * ---------------------------------------------------------------------------
 * Cette fonction est appele au chargement, elle doit retourner
 * TRUE pour que sa fonction de filtrage soit invoque.
 * ---------------------------------------------------------------------------
 */
BOOLEAN WINAPI InitializeChangeNotify(void)
{
	// Si vous faites a ici: tant pis pour vous !
	// car la DLL est charge au dmarrage: la Mbox ne sera pas visible
	// et donc provoquera un blocage infini !!!!
	//MessageBox(NULL,"InitializeChang	eNotify","syncunlock",MB_OK);

	Log(EVENTLOG_INFORMATION_TYPE,_T("Appel de InitializeChangeNotify()"));

	return TRUE;
}

#if 0 // pas utilise par notre DLL
/*
 * ---------------------------------------------------------------------------
 * Fonction de filtrage qui permet d'accepter ou refuser 
 * les mots de passe. 
 *
 * Nous ne filtrons pas les mots de passe: on renvoie toujours OK
 * ---------------------------------------------------------------------------
 */
BOOLEAN WINAPI PasswordFilter(
							PUNICODE_STRING UserName,
							PUNICODE_STRING FullName,
							PUNICODE_STRING NewPassword,
							BOOLEAN SetOperation)
{
	Log(EVENTLOG_INFORMATION_TYPE,"Appel de PasswordFilter()");

	return TRUE;
}
#endif

/*
 * ---------------------------------------------------------------------------
 * C'est la fonction principale: elle est appele par le LSA quand le
 * nouveau mot de passe a t accept par toutes les DLL de filtrage et modifi.
 *
 * Notre objectif est d'invoquer un pgm externe en lui passant le nom du compte
 * en paramtre
 * ---------------------------------------------------------------------------
 */
NTSTATUS WINAPI PasswordChangeNotify(
								PUNICODE_STRING UserName,	/* nom de l'utilisateur */
								ULONG RelativeId,			/* son R-ID */
								PUNICODE_STRING NewPassword	/* son nouveau mot de passe */
								)
{
	DWORD	status;
	lsParm	parm;
	TCHAR	szMatricule[128];
	TCHAR	szPassword[128];
	TCHAR	cLastChar;

	Log(EVENTLOG_INFORMATION_TYPE,_T("Appel de PasswordChangeNotify()"));

	/* Lit les paramtres de configuration dans la base de registres: */
	if (status = GetParms(&parm,Log)) {
		Log(EVENTLOG_ERROR_TYPE,_T("GetParm a chou: %d"),status);
		return 1;
	}

	/* Rien  faire si la DLL n'est pas activ: */
	if (!parm.m_active) {
		Log(EVENTLOG_INFORMATION_TYPE,_T("syncunlock n'est pas activ !"));
		return 0;
	}

	/* Convertit les chaines UNICODE en ANSI: */
	WideCharToMultiByte(CP_ACP,0,UserName->Buffer,UserName->Length,szMatricule,sizeof(szMatricule),NULL,NULL);
	szMatricule[UserName->Length / sizeof(WCHAR)] = _T('\0');
	WideCharToMultiByte(CP_ACP,0,NewPassword->Buffer,NewPassword->Length,szPassword,sizeof(szPassword),NULL,NULL);
	szPassword[NewPassword->Length / sizeof(WCHAR)] = _T('\0');

	/*
	 * ignore les noms qui finissent par un '$' (noms de machine !)
	 */
	cLastChar = szMatricule[_tcslen(szMatricule)-1];
	if (cLastChar == _T('$')) {
		Log(EVENTLOG_INFORMATION_TYPE,_T("Ignore les noms de machine"));
		return TRUE;
	}

	Start(&parm,szMatricule);

#if 0
	/* Recopie le nouveau fichier ID dans le rpertoire d'accueil de l'utilisateur: */
	status = NetUserGetInfo(	
							NULL,			/* recherche sur la machine locale */
							UserName->Buffer,
							1,				/* niveau d'information requis */
							&pUserInfo);	/* informations */
	if (status != NOERROR) {
		Log(EVENTLOG_ERROR_TYPE,_T("NetUserGetInfo a chou: %d",status));
		DeleteFile(parm.m_szTempFileName); /* ajout par JD le 07/09/05 */
		return status;
	}

	/* Libere le buffer d'info: */
	NetApiBufferFree(pUserInfo);
#endif

	return TRUE;
}

/* EOF */
